/*** -*- Mode: Javascript; tab-width: 2; 
The contents of this file are subject to the Mozilla Public
License Version 1.1 (the "License"); you may not use this file
except in compliance with the License. You may obtain a copy of
the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS
IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
implied. See the License for the specific language governing
rights and limitations under the License.

The Original Code is jslib code.
The Initial Developer of the Original Code is jslib team.

Portions created by jslib team are
Copyright (C) 2000 jslib team.  All
Rights Reserved.

Contributor(s): Pete Collins, 
                Doug Turner, 
                Brendan Eich, 
                Warren Harris, 
                Eric Plaster,
                Martin Kutschker

The purpose of this file is to make it a little easier to use 
xpcom nsIFile file IO library from js

 su_File API 
    file.js

 Base Class:
    su_FileSystem
      filesystem.js

 Function List:
    // Constructor
    su_File(aPath)                         creates the su_File object and sets the file path

    // file stream methods
    open(aMode, aPermissions);          open a file handle for reading,
                                        writing or appending.  permissions are optional.
    read();                             returns the contents of a file
    readline();                         returns the next line in the file.
    EOF;                                boolean check 'end of file' status
    write(aContents);                   writes the contents out to file.
    copy(aDest);                        copy the current file to a aDest
    close();                            closes a file handle
    create();                           creates a new file if one doesn't already exist
    exists();                           check to see if a file exists

    // file attributes
    size;                               read only attribute gets the file size
    ext;                                read only attribute gets a file extension if there is one
    permissions;                        attribute gets or sets the files permissions
    dateModified;                       read only attribute gets last modified date in locale string

    // file path attributes
    leaf;                               read only attribute gets the file leaf
    path;                               read only attribute gets the path
    parent;                             read only attribute gets parent dir part of a path

    // direct manipulation
    nsIFile                             returns an nsIFile obj

    // utils
    remove();                           removes the current file
    append(aLeaf);                      appends a leaf name to the current file
    appendRelativePath(aRelPath);       appends a relitave path the the current file

    // help!
    help;                               currently dumps a list of available functions

 Instructions:

       First su_include this js file in your xul file.  
       Next, create an su_File object:

          var file = new su_File("/path/file.ext");

       To see if the file exists, call the exists() member.  
       This is a good check before going into some
       deep code to try and extract information from a non-existant file.

       To open a file for reading<"r">, writing<"w"> or appending<"a">,
       just call:

          file.open("w", 0644);

       where in this case you will be creating a new file called '/path/file.ext', 
       with a mode of "w" which means you want to write a new file.

       If you want to read from a file, just call:

          file.open(); or
          file.open("r");
          var theFilesContents    = file.read();

          ---- or ----

          while(!file.EOF) {
            var theFileContentsLine = file.readline();
            dump("line: "+theFileContentsLine+"\n");
          }

       The file contents will be returned to the caller so you can do something usefull with it.

          file.close();

       Calling 'close()' destroys any created objects.  If you forget to use file.close() no probs
       all objects are discarded anyway.

       Warning: these API's are not for religious types

************/

// ensure jslib base is loaded
if (typeof(su_JS_LIB_LOADED)=='boolean') {

// test to make sure filesystem base class is loaded
if (typeof(su_JS_FILESYSTEM_LOADED)!='boolean')
  su_include(su_jslib_filesystem);

/****************** Globals **********************/
const su_JS_FILE_LOADED           = true;
const su_JS_FILE_FILE             = "file.js";

const su_JS_FILE_IOSERVICE_CID    = "@mozilla.org/network/io-service;1";
const su_JS_FILE_I_STREAM_CID     = "@mozilla.org/scriptableinputstream;1";
const su_JS_FILE_OUTSTREAM_CID    = "@mozilla.org/network/file-output-stream;1";
const su_JS_FILE_BINOUTSTREAM_CID = "@mozilla.org/binaryoutputstream;1";
const su_JS_FILE_BININSTREAM_CID  = "@mozilla.org/binaryinputstream;1";

const su_JS_FILE_F_TRANSPORT_SERVICE_CID  = "@mozilla.org/network/file-transport-service;1";

const su_JS_FILE_I_IOSERVICE              = su_jslibI.nsIIOService;
const su_JS_FILE_I_SCRIPTABLE_IN_STREAM   = "nsIScriptableInputStream";
const su_JS_FILE_I_FILE_OUT_STREAM        = su_jslibI.nsIFileOutputStream;
const su_JS_FILE_I_BINARY_OUT_STREAM      = "nsIBinaryOutputStream";
const su_JS_FILE_I_BINARY_IN_STREAM       = "nsIBinaryInputStream";

const su_JS_FILE_READ          = 0x01;  // 1
const su_JS_FILE_WRITE         = 0x08;  // 8
const su_JS_FILE_APPEND        = 0x10;  // 16

const su_JS_FILE_READ_MODE     = "r";
const su_JS_FILE_WRITE_MODE    = "w";
const su_JS_FILE_APPEND_MODE   = "a";
const su_JS_FILE_BINARY_MODE   = "b";

const su_JS_FILE_FILE_TYPE     = 0x00;  // 0

const su_JS_FILE_CHUNK         = 1024;  // buffer for readline => set to 1k

const su_JS_FILE_DEFAULT_PERMS = 0644;

const su_JS_FILE_OK            = true;

try {
  const su_JS_FILE_InputStream  = new Components.Constructor
  (su_JS_FILE_I_STREAM_CID, su_JS_FILE_I_SCRIPTABLE_IN_STREAM);

  const su_JS_FILE_IOSERVICE    = su_jslibGetService(su_JS_FILE_IOSERVICE_CID,
                                 su_JS_FILE_I_IOSERVICE);

} catch (e) {
  su_jslibError(e);
}

/***
 * Possible values for the ioFlags parameter 
 * From: 
 * http://lxr.mozilla.org/seamonkey/source/nsprpub/pr/su_include/prio.h#601
 */


// #define PR_RDONLY       0x01
// #define PR_WRONLY       0x02
// #define PR_RDWR         0x04
// #define PR_CREATE_FILE  0x08
// #define PR_APPEND       0x10
// #define PR_TRUNCATE     0x20
// #define PR_SYNC         0x40
// #define PR_EXCL         0x80

const su_JS_FILE_NS_RDONLY               = 0x01;
const su_JS_FILE_NS_WRONLY               = 0x02;
const su_JS_FILE_NS_RDWR                 = 0x04;
const su_JS_FILE_NS_CREATE_FILE          = 0x08;
const su_JS_FILE_NS_APPEND               = 0x10;
const su_JS_FILE_NS_TRUNCATE             = 0x20;
const su_JS_FILE_NS_SYNC                 = 0x40;
const su_JS_FILE_NS_EXCL                 = 0x80;
/****************** Globals **********************/


/****************************************************************
* void su_File(aPath)                                              *
*                                                               *
* class constructor                                             *
* aPath is an argument of string local file path                *
* returns NS_OK on success, exception upon failure              *
*   Ex:                                                         *
*     var p = '/tmp/foo.dat';                                   *
*     var f = new su_File(p);                                      *
*                                                               *
*   outputs: void(null)                                         *
****************************************************************/

function su_File (aPath) {

  var rv;
  if (!aPath)
    return su_jslibErrorMsg("NS_ERROR_INVALID_ARG");

  // if the argument is a su_File or nsIFile object
  if (su_jslibTypeIsObj(aPath))
    rv = aPath.path;
  else
    rv = arguments;

  return this.initPath(rv);
} // constructor

su_File.prototype = new su_FileSystem();

// member vars
su_File.prototype.mMode        = null;
su_File.prototype.mIsBinary    = false;
su_File.prototype.mFileChannel = null;
su_File.prototype.mTransport   = null;
su_File.prototype.mURI         = null;
su_File.prototype.mOutStream   = null;
su_File.prototype.mInputStream = null;
su_File.prototype.mLineBuffer  = null;
su_File.prototype.mPosition    = 0;

/********************* OPEN *************************************
* bool open(aMode, aPerms)                                      *
*                                                               *
* opens a file handle to read, write or append                  *
* aMode is an argument of string 'w', 'a', 'r'                  *
* returns true on success, null on failure                      *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*                                                               *
*   outputs: void(null)                                         *
****************************************************************/

su_File.prototype.open = function (aMode, aPerms) 
{

  // close any existing file handles
  this.close();

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.mPath) 
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (this.exists() && this.mFileInst.isDirectory()) 
      return su_jslibErrorMsg("NS_ERROR_FILE_IS_DIRECTORY");

  if (this.mMode) 
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.mURI) {
    if (!this.exists())
      this.create();
    this.mURI = su_JS_FILE_IOSERVICE.newFileURI(this.mFileInst);
  }

  if (!aMode)
    aMode=su_JS_FILE_READ_MODE;

  this.resetCache();
  var rv;
  
  this.mIsBinary = false;
  var access;
  while (aMode.length > 0) {
    switch (aMode[0]) {
	  case su_JS_FILE_WRITE_MODE:
	  case su_JS_FILE_APPEND_MODE:
	  case su_JS_FILE_READ_MODE: {
	    access = aMode[0];
		break;
	  }
	  case su_JS_FILE_BINARY_MODE: {
	    this.mIsBinary = true;
	    break;
	  }
	  default:
	    return su_jslibErrorMsg("NS_ERROR_INVALID_ARG");
	}
	aMode = aMode.substring(1);
  }
  aMode = access;

  switch (aMode) 
  {
    case su_JS_FILE_WRITE_MODE: 
    case su_JS_FILE_APPEND_MODE: {
      try {
        if (!this.mFileChannel)
          this.mFileChannel = su_JS_FILE_IOSERVICE.newChannelFromURI(this.mURI);
      } catch (e)    {
        return su_jslibError(e);
      }    

      if (aPerms && !this.validatePermissions(aPerms))
        su_jslibErrorMsg("NS_ERROR_INVALID_ARG");

      if (!aPerms)
        aPerms=su_JS_FILE_DEFAULT_PERMS;

      try {
        var offSet=0;
        if (aMode == su_JS_FILE_WRITE_MODE) {
          this.mMode=su_JS_FILE_WRITE_MODE;
          // create a filestream
          var fs = su_jslibCreateInstance(su_JS_FILE_OUTSTREAM_CID, su_JS_FILE_I_FILE_OUT_STREAM);

          fs.init(this.mFileInst, su_JS_FILE_NS_TRUNCATE | 
                                  su_JS_FILE_NS_WRONLY, 00004, null); 
          this.mOutStream = fs;
        } else {
          this.mMode=su_JS_FILE_APPEND_MODE;
          // create a filestream
          fs = su_jslibCreateInstance(su_JS_FILE_OUTSTREAM_CID, su_JS_FILE_I_FILE_OUT_STREAM);

          fs.init(this.mFileInst, su_JS_FILE_NS_RDWR | 
                                  su_JS_FILE_NS_APPEND, 00004, null); 
          this.mOutStream = fs;
        }
        if (this.mIsBinary) {
          // wrap a nsIBinaryOutputStream around the actual file
          var binstream = su_jslibCreateInstance(su_JS_FILE_BINOUTSTREAM_CID, su_JS_FILE_I_BINARY_OUT_STREAM);
          binstream.setOutputStream(this.mOutStream);
          this.mOutStream = binstream;
        }
      } catch (e) {
        return su_jslibError(e);
      }
      rv = su_JS_LIB_OK;
      break;
    }

    case su_JS_FILE_READ_MODE: {
      if (!this.exists()) 
        su_jslibErrorMsg("NS_ERROR_FILE_NOT_FOUND");

      this.mMode=su_JS_FILE_READ_MODE;

      try {
        su_jslibDebug('****** '+this.mURI);
        this.mFileChannel = su_JS_FILE_IOSERVICE.newChannelFromURI(this.mURI);
        this.mLineBuffer  = new Array();
        if (this.mIsBinary) {
          // wrap a nsIBinaryInputStream around the nsIInputStream
          this.mInputStream = su_jslibCreateInstance(su_JS_FILE_BININSTREAM_CID, su_JS_FILE_I_BINARY_IN_STREAM);
          this.mInputStream.setInputStream(this.mFileChannel.open());
        } else {
          // wrap a nsIScriptableInputStream around the nsIInputStream
          this.mInputStream = new su_JS_FILE_InputStream();    
          this.mInputStream.init(this.mFileChannel.open());
        }
        rv = su_JS_LIB_OK;
      } catch (e) {
        rv = su_jslibError(e);
      }
      break;
    }

    default:
      rv = su_jslibErrorMsg("NS_ERROR_INVALID_ARG");
  }
  return rv;
}

/********************* READ *************************************
* string read()                                                 *
*                                                               *
* reads a file if the file is binary it will                    *
* return type ex: ELF                                           *
* takes no arguments needs an open read mode filehandle         *
* returns string on success, null on failure                    *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.open();                                                *
*     f.read();                                                 *
*                                                               *
*   outputs: <string contents of foo.dat>                       *
****************************************************************/

su_File.prototype.read = function (aSize) 
{

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (this.mMode != su_JS_FILE_READ_MODE) {
    this.close();
    return su_jslibErrorMsg("NS_ERROR_NOT_AVAILABLE");
  }

  var rv = null;
  try {
    if (!this.mFileInst || !this.mInputStream) 
      su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

    if (this.mIsBinary) {
      rv = this.mInputStream.readByteArray(aSize != undefined ? aSize : this.mFileInst.fileSize);
    } else {
      rv = this.mInputStream.read(aSize != undefined ? aSize : this.mFileInst.fileSize);
    }
    this.mInputStream.close();
  } catch (e) { 
    rv = su_jslibError(e);
  }
  return rv;
}

/********************* READLINE**********************************
* string readline()                                             *
*                                                               *
* reads a file if the file is binary it will                    *
* return type string                                            *
* takes no arguments needs an open read mode filehandle         *
* returns string on success, null on failure                    *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.open();                                                 *
*     while(!f.EOF)                                             *
*       dump("line: "+f.readline()+"\n");                       *
*                                                               *
*   outputs: <string line of foo.dat>                           *
****************************************************************/

su_File.prototype.readline = function ()
{

  if (!this.checkInst() || !this.mInputStream)
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  var rv      = null;
  var buf     = null;
  var tmp     = null;
  try {
    if (this.mLineBuffer.length < 2) {
      buf = this.mInputStream.read(su_JS_FILE_CHUNK);
      this.mPosition = this.mPosition + su_JS_FILE_CHUNK;
      if (this.mPosition > this.mFileInst.fileSize) 
        this.mPosition  = this.mFileInst.fileSize;
      if (buf) {
        if (this.mLineBuffer.length == 1) {
          tmp = this.mLineBuffer.shift();
          buf = tmp+buf;
        }
        this.mLineBuffer = buf.split(/[\n\r]/);
      }
    }
    rv = this.mLineBuffer.shift();
  } catch (e) {
    rv = su_jslibError(e);
  }
  return rv;
}

/********************* EOF **************************************
* bool getter EOF()                                             *
*                                                               *
* boolean check 'end of file' status                            *
* return type boolean                                           *
* takes no arguments needs an open read mode filehandle         *
* returns true on eof, false when not at eof                    *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.open();                                                 *
*     while(!f.EOF)                                             *
*       dump("line: "+f.readline()+"\n");                       *
*                                                               *
*   outputs: true or false                                      *
****************************************************************/

su_File.prototype.__defineGetter__('EOF', 
function ()
{
  if (!this.checkInst() || !this.mInputStream)
    throw su_jslibErrorThrow("NS_ERROR_NOT_INITIALIZED");

  if ((this.mLineBuffer.length > 0) || (this.mInputStream.available() > 0)) 
    return false;
  
  return true;
})

/********************* WRITE ************************************
* write()                                                       *
*                                                               *
*  Write data to a file                                         *
*                                                               *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.open("w");                                              *
*     f.write();                                                *
*                                                               *
*   outputs: su_JS_LIB_OK upon success                             *
****************************************************************/

su_File.prototype.write = function (aBuffer)
{

  if (!aBuffer)
    return su_jslibErrorMsg("NS_ERROR_INVALID_ARG");

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (this.mMode == su_JS_FILE_READ_MODE) {
    this.close();
    su_jslibErrorMsg("NS_ERROR_FILE_READ_ONLY");
  }

  if (!this.mFileInst)
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED")

  var rv = null;
  try {
    if (this.mIsBinary && aBuffer.constructor == Array) {
      this.mOutStream.writeByteArray(aBuffer, aBuffer.length);
    } else {
      this.mOutStream.write(aBuffer, aBuffer.length);
    }
    this.mOutStream.flush();
    rv = su_JS_LIB_OK;
  } catch (e) { 
    rv = su_jslibError(e);
  }

  return rv;
}

/********************* COPY *************************************
* void copy(aDest)                                              *
*                                                               *
* void file close                                               *
* return type void(null)                                        *
* takes no arguments closes an open file stream and             *
* deletes member var instances of objects                       *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     fopen();                                                  *
*     f.close();                                                *
*                                                               *
*   outputs: su_JS_LIB_OK upon success                             *
****************************************************************/

su_File.prototype.copy = function (aDest)
{

  if (!aDest)
    return su_jslibErrorMsg("NS_ERROR_INVALID_ARG");

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.exists()) 
    return su_jslibErrorMsg("NS_ERROR_FILE_NOT_FOUND");

  var rv = su_JS_LIB_OK;
  try {
    var dest = new su_JS_FS_File_Path(aDest);
    var copyName, dir = null;

    if (dest.equals(this.mFileInst)) 
      return su_jslibErrorMsg("NS_ERROR_FILE_COPY_OR_MOVE_FAILED");

    if (dest.exists()) 
      return su_jslibErrorMsg("NS_ERROR_FILE_ALREADY_EXISTS");

    if (this.mFileInst.isDirectory()) 
      return su_jslibErrorMsg("NS_ERROR_FILE_IS_DIRECTORY");

    if (!dest.exists()) {
      copyName = dest.leafName;
      dir = dest.parent;

      if (!dir.exists()) 
        return su_jslibErrorMsg("NS_ERROR_FILE_NOT_FOUND");

      if (!dir.isDirectory()) 
        return su_jslibErrorMsg("NS_ERROR_FILE_DESTINATION_NOT_DIR");
    }

    if (!dir) {
      dir = dest;
      if (dest.equals(this.mFileInst)) 
        return su_jslibErrorMsg("NS_ERROR_FILE_COPY_OR_MOVE_FAILED");
    }
    this.mFileInst.copyTo(dir, copyName);
    su_jslibDebug(su_JS_FILE_FILE+":copy successful!");
  } catch (e) {
    rv = su_jslibError(e);
  }
  return rv;
}

/********************* CLOSE ************************************
* void close()                                                  *
*                                                               *
* void file close                                               *
* return type void(null)                                        *
* takes no arguments closes an open file stream and             *
* deletes member var instances of objects                       *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     fopen();                                                  *
*     f.close();                                                *
*                                                               *
*   outputs: void(null)                                         *
****************************************************************/

su_File.prototype.close = function () 
{
  /***************** Destroy Instances *********************/
  if (this.mFileChannel)   delete this.mFileChannel;
  if (this.mInputStream)   delete this.mInputStream;
  if (this.mTransport)     delete this.mTransport;
  if (this.mMode)          this.mMode = null;
  if (this.mOutStream) {
    this.mOutStream.close();
    delete this.mOutStream;
  }
  if (this.mInputStream) {
    this.mInputStream.close();
    delete this.mInputStream;
  }
  if (this.mLineBuffer)    this.mLineBuffer = null;
  this.mPosition           = 0;
  /***************** Destroy Instances *********************/

  return void(null);
}

/********************* CREATE *****************************/
su_File.prototype.create = function ()
{

  // We can probably implement this so that it can create a 
  // file or dir if a long non-existent mPath is present

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (this.exists()) 
    return su_jslibErrorMsg("NS_ERROR_FILE_ALREADY_EXISTS");

  var rv = null;
  try { 
    rv = this.mFileInst.create(su_JS_FILE_FILE_TYPE, su_JS_FILE_DEFAULT_PERMS); 
  } catch (e) {
    rv = su_jslibError(e);
  }

  return rv;
}

/********************* REMOVE *******************************/
su_File.prototype.remove = function ()
{
  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.mPath) 
    return su_jslibErrorMsg("NS_ERROR_FILE_INVALID_PATH");

  this.close();
  var rv;
  try {
    // this is a non recursive remove because we are only dealing w/ files.
    rv = this.mFileInst.remove(false); 
  } catch (e) {
    rv = su_jslibError(e);
  }

  return rv;
}

/********************* POS **************************************
* int getter POS()                                              *
*                                                               *
* int file position                                             *
* return type int                                               *
* takes no arguments needs an open read mode filehandle         *
* returns current position, default is 0 set when               *
* close is called                                               *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.open();                                                 *
*     while(!f.EOF){                                            *
*       dump("pos: "+f.pos+"\n");                               *
*       dump("line: "+f.readline()+"\n");                       *
*     }                                                         *
*                                                               *
*   outputs: int pos                                            *
****************************************************************/

su_File.prototype.__defineGetter__('pos', function (){ return this.mPosition; })

/********************* SIZE *************************************
* int getter size()                                             *
*                                                               *
* int file size                                                 *
* return type int                                               *
* takes no arguments a getter only                              *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.size;                                                   *
*                                                               *
*   outputs: int 16                                             *
****************************************************************/

su_File.prototype.__defineGetter__('size',
function ()
{

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.mPath) 
    return su_jslibErrorMsg("NS_ERROR_FILE_INVALID_PATH");

  if (!this.exists()) 
    return su_jslibErrorMsg("NS_ERROR_FILE_NOT_FOUND");

  var rv = null;
  this.resetCache();
  try { 
    rv=this.mFileInst.fileSize; 
  } catch(e) {
    rv = su_jslibError(e);
  }

  return rv;
}) // END size Getter

/********************* EXTENSION ********************************
* string getter ext()                                           *
*                                                               *
* string file extension                                         *
* return type string                                            *
* takes no arguments a getter only                              *
*   Ex:                                                         *
*     var p='/tmp/foo.dat';                                     *
*     var f=new su_File(p);                                        *
*     f.ext;                                                    *
*                                                               *
*   outputs: dat                                                *
****************************************************************/

su_File.prototype.__defineGetter__('ext', 
function ()
{

  if (!this.checkInst())
    return su_jslibErrorMsg("NS_ERROR_NOT_INITIALIZED");

  if (!this.mPath) 
    return su_jslibErrorMsg("NS_ERROR_FILE_INVALID_PATH");
  
  if (!this.exists()) 
    return su_jslibErrorMsg("NS_ERROR_FILE_NOT_FOUND");

  var rv = null;
  try {
    var leafName  = this.mFileInst.leafName;
    var dotIndex  = leafName.lastIndexOf('.');
    rv=(dotIndex >= 0) ? leafName.substring(dotIndex+1) : "";
  } catch(e) {
    rv = su_jslibError(e);
  }

  return rv;
}) // END ext Getter

su_File.prototype.super_help = su_FileSystem.prototype.help;

/********************* HELP *****************************/
su_File.prototype.__defineGetter__('help', 
function ()
{
  const help = this.super_help()            +
    "   open(aMode);\n"                     +
    "   read();\n"                          +
    "   readline();\n"                      +
    "   EOF;\n"                             +
    "   write(aContents, aPermissions);\n"  +
    "   copy(aDest);\n"                     +
    "   close();\n"                         +
    "   create();\n"                        +
    "   remove();\n"                        +
    "   size;\n"                            +
    "   ext;\n"                             +
    "   help;\n";

  return help;
})

su_jslibDebug('*** load: '+su_JS_FILE_FILE+' OK');

} // END BLOCK su_JS_LIB_LOADED CHECK

// If su_jslib base library is not loaded, dump this error.
else
{
    dump("su_JS_FILE library not loaded:\n"                                +
         " \tTo load use: chrome://su_jslib/content/su_jslib.js\n"            +
         " \tThen: su_include(su_jslib_file);\n\n");
}
